@@ -33,42 +33,50 @@ def show_fig(image, title=None, color='Spectral', ctitle='', bounds=None, res=No
     if savefig == True:
     	fig.savefig(title + '.jpg', dpi=500) # to save the plot as a jpg image
 
-def scatterplot(data, xtitle, ytitle, title, colors, labels, markers, xyLine=False, buff=100, savefig=False):
+def scatterplot(data, xtitle, ytitle, title, colors, labels, markers='.', alpha=0.7, xyLine=False, 
+                xbuff=1, ybuff=100, leg_loc='best', anchor=None, savefig=False):
     """
-	Create a scatterplot from data
-		data: input data (list of arrays of length 2) (e.g. [[0, 0, 0, 0], [1, 2, 3, 4]] or [[[0, 0, 0], [1, 2, 3]], [[1, 1, 1, 1], [1, 2, 3, 4]]])
-		xtitle, ytitle: axis titles (str)
-		colors: list of colors for each x-y scatterplot dataset (list of str, same length as data)
-		labels: legend label for data (list of str, same length as data) (e.g. ['label1', 'label2'])
-		markers: scatterplot marker (list of str, same length as data)
-		xyLine: to show the 1-to-1 line (boolean)
-		buff: buffer on axis limits (int or float)
+    Create a scatterplot from data
+        data: input data (list of arrays of length 2)
+        xtitle, ytitle: axis titles (str)
+        colors: list of colors for each x-y scatterplot dataset (list of str, same length as data)
+        labels: legend label for data (list of str, same length as data) (e.g. ['label1', 'label2'])
+        markers: scatterplot marker (list of str, same length as data, or string if all markers should be the same)
+        alpha: marker alpha value; transparency (float ranging 0 to 1)
+        xyLine: to show the 1-to-1 line (boolean)
+        leg_loc: location of the legend (str; upper lower center left right middle)
+        buff: buffer on axis limits (int or float)
     """
     fig = plt.figure()
     ax = fig.add_subplot(111)
     plt.grid()
+    
+    if type(markers) == str:
+        markers = [markers] * len(data)
 
     for i, xy in enumerate(data):
-        ax.scatter(xy[0], xy[1], c=colors[i], alpha=0.7, marker=markers[i], label=labels[i])
+        ax.scatter(xy[0], xy[1], c=colors[i], alpha=alpha, marker=markers[i], label=labels[i])
     ax.set_xlabel(xtitle)
     ax.set_ylabel(ytitle)
-    ax.legend()
+    ax.legend(loc=leg_loc, bbox_to_anchor=anchor)
     if xyLine == True:
         ax.axline([0, 0], [1, 1], c='k', ls='--', lw=1)
 
-    maximums = []
-    minimums = []
-    for d in np.array(data):
-        maximums.append(max([max(d[0]), max(d[1])]) + buff)
-        minimums.append(min([min(d[0]), min(d[1])]) - buff)
+    maximums_x, minimums_x, maximums_y, minimums_y = [], [], [], []
+    for d in np.array(data, dtype=object):
+        maximums_x.append(np.nanmax(d[0]))
+        minimums_x.append(np.nanmin(d[0]))
+        maximums_y.append(np.nanmax(d[1]))
+        minimums_y.append(np.nanmin(d[1]))
 
-    ax.set_xlim([min(minimums), max(maximums)])
-    ax.set_ylim([min(minimums), max(maximums)])
-    fig.suptitle(title, size=8)
+    ax.set_xlim([min(minimums_x) - xbuff, max(maximums_x) + xbuff])
+    ax.set_ylim([min(minimums_y) - ybuff, max(maximums_y) + ybuff])
+    fig.suptitle(title, size=12)
     
     if savefig == True:
-    	figName = title.replace(' ', '_') + '.png'
-    	fig.savefig(figName, dpi=250)
+        figName = title.replace(' ', '_') + '.png'
+        fig.savefig(figName, dpi=250)
+    return fig, ax
 
 
 def plotMany(cbarTitle, color, plotTitle, *kwargs):
@@ -551,8 +559,24 @@ def plotData6(dataVals1, cbarTitle1, color1, title1, dataVals2, cbarTitle2, colo
     else:
     	plt.show()
 
-def plotDataPoints(dataVals1, cbarTitle1, color1, dataVals2, cbarTitle2, color2,
-                   plotTitle, pointx, pointy, pointlabels, savefig=False):
+def plotDataPoints(dataVals, cbarTitle, color, plotTitle, pointx, pointy, pointlabels):
+    # Plot a data array with annotated points
+    fig = plt.figure(figsize=(10, 6))
+    ax = fig.add_subplot(111)
+
+    dataVals = dataVals.astype(float)
+    dataVals[dataVals == 0] = np.nan            # to remove 0 values for white background in plot
+    im = ax.imshow(dataVals, cmap=plt.cm.get_cmap(color))  # check len datavals
+    fig.colorbar(im, label=cbarTitle)
+    ax.scatter(pointx, pointy, s=6, c='Red', marker='o')        # add points where reference data exists
+    for i, txt in enumerate(pointlabels):                       # add labels to each point
+        ax.annotate(text=txt, xy=(pointx[i]+3, pointy[i]-3), c='Red', size=18)
+
+    fig.suptitle(plotTitle, ha='left')
+    fig.tight_layout(pad=3, w_pad=0.0, h_pad=3.0)
+    plt.show()
+
+def plotDataPoints2(dataVals1, cbarTitle1, color1, dataVals2, cbarTitle2, color2, plotTitle, pointx, pointy, pointlabels, savefig=False):
     '''
     Plots a map from input values (array-like) with labelled point locations from reference data
     Plots 2 side by side maps: identical but with difference basemaps (e.g. elevation and pixel mass balance)
@@ -601,7 +625,7 @@ def plotDataPoints(dataVals1, cbarTitle1, color1, dataVals2, cbarTitle2, color2,
     else:
     	plt.show()
 
-def velPlot(vx, vy, v_tot, area, threshold):
+def velPlot(vx, vy, v_tot, pixel_size, threshold):
     '''
     Show velocity vector arrows
     :param vx1: x-direction velocity
@@ -612,7 +636,7 @@ def velPlot(vx, vy, v_tot, area, threshold):
     :return:
     '''
     # fig, ax = plt.subplots()                                  # uncomment lines to plot this graph separately
-    if area/1000000 > 10:
+    if pixel_size < 10:
         freq = 20
     else:
         freq = 10
